//+------------------------------------------------------------------+
//|                                      Price Action Test (Z-score) |
//|                                                    EarnForex.com |
//|                                        https://www.earnforex.com |
//|            							                       2010-2021 |
//+------------------------------------------------------------------+
#property copyright "www.EarnForex.com, 2010-2021"
#property link      "https://www.earnforex.com/guides/z-score-optimization-in-forex-trading/"
#property version   "1.01"

#property description "Trades using trailing stop with reverse. Initial direction is random."
#property description " + Fixed ratio position sizing."
#property description " + Z-Score optimization."

#include <Trade/Trade.mqh>
#include <Trade/PositionInfo.mqh>

input int StopLoss = 100; // Isn't used
input int TakeProfit = 0;
input int Slippage = 100; 	// Tolerated slippage in pips, pips are fractional
input double iLots = 0.1;
input int ATRper = 24;
input int Mute = true; // If true, no extra output to journal is made

// Main trading objects
CTrade *Trade;
CPositionInfo PositionInfo;

// Global variables
bool HaveLongPosition;
bool HaveShortPosition;
double SL, TP;
ENUM_POSITION_TYPE LastPosition = -1;
int random;
double Lots;
int myATR;

bool TradeBlock = false;
ENUM_POSITION_TYPE VirtualDirection;
bool VirtualOpen = false;
double VirtualOP = 0;
double VirtualSL = 0;

//+------------------------------------------------------------------+
//| Expert Initialization Function                                   |
//+------------------------------------------------------------------+
void OnInit()
{
	// Initialize the Trade class object
	Trade = new CTrade;
	Trade.SetDeviationInPoints(Slippage);
	MathSrand((int)TimeCurrent());
  	random = MathRand();
   myATR = iATR(NULL, 0, ATRper);
}

//+------------------------------------------------------------------+
//| Expert Deinitialization Function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
	delete Trade;
}

//+------------------------------------------------------------------+
//| Expert Every Tick Function                                       |
//+------------------------------------------------------------------+
void OnTick()
{
	if (TerminalInfoInteger(TERMINAL_TRADE_ALLOWED) == false) return;
	
   if (!TradeBlock)
   {
      // Looking back for 1 week, but in fact, 1 day would be enough
      HistorySelect(TimeCurrent() - 7 * 24 * 3600, TimeCurrent());
      ulong deal = HistoryDealsTotal();
      deal = HistoryDealGetTicket((int)deal - 1);
      if (deal > 0)
      {
         if (HistoryDealGetInteger(deal, DEAL_ENTRY) == DEAL_ENTRY_OUT)
         {
            if (HistoryDealGetDouble(deal, DEAL_PROFIT) > 0)
            {
               TradeBlock = true;
               if (!Mute) Print("Real trading blocked on: ", deal, " ", HistoryDealGetInteger(deal, DEAL_ENTRY), " ", HistoryDealGetDouble(deal, DEAL_PROFIT));
            }
         }
      }
   }
   else if (VirtualOpen)
   {
      if (VirtualSL != 0) // Track stop-loss trigger
      {
         if (VirtualDirection == POSITION_TYPE_BUY)
         {
            double Bid = SymbolInfoDouble(Symbol(), SYMBOL_BID);
            if (Bid <= VirtualSL)
            {
               if (!Mute) Print("Virtual SL Triggered.");
               ClosePrevious();
            }
   
         }
         else if (VirtualDirection == POSITION_TYPE_SELL)
         {
          	double Ask = SymbolInfoDouble(Symbol(), SYMBOL_ASK);
          	if (Ask >= VirtualSL)
          	{
          	   if (!Mute) Print("Virtual SL Triggered.");
          	   ClosePrevious();
          	}
         }
      }
   }

   // Getting the ATR values
   double ATR[1];
   if (CopyBuffer(myATR, 0, 0, 1, ATR) != 1) return;
   // You can uncomment this line if you want to use ATR-based position sizing
   //Lots = NormalizeDouble(0.01 * AccountInfoDouble(ACCOUNT_BALANCE) / (ATR[0] * 100000), 1);
   // Reverse-Martingale position sizing
   Lots = NormalizeDouble(AccountInfoDouble(ACCOUNT_BALANCE) / 10000 * 1.5, 1);
   ATR[0] *= 3;

   if (ATR[0] <= (SymbolInfoInteger(Symbol(), SYMBOL_TRADE_STOPS_LEVEL) + SymbolInfoInteger(Symbol(), SYMBOL_SPREAD)) * _Point) ATR[0] = (SymbolInfoInteger(Symbol(), SYMBOL_TRADE_STOPS_LEVEL) + SymbolInfoInteger(Symbol(), SYMBOL_SPREAD)) * _Point;

  	// Check what position is currently open
 	GetPositionStates();
   
 	// Adjust SL and TP of the current position
 	if ((HaveLongPosition) || (HaveShortPosition)) AdjustSLTP(ATR[0]);
   else
	{
      double Ask, Bid;
   
   	// Buy condition
   	if (((LastPosition == -1) && (random % 2 == 0)) || (LastPosition == POSITION_TYPE_SELL))
   	{
  			for (int i = 0; i < 10; i++)
  			{
  		   	Ask = SymbolInfoDouble(Symbol(), SYMBOL_ASK);
  				Bid = SymbolInfoDouble(Symbol(), SYMBOL_BID);
  				// Bid and Ask are swapped to preserve the probabilities and decrease/increase profit/loss size
  		   	SL = NormalizeDouble(Bid - ATR[0], _Digits);
  		   	if (TakeProfit) TP = NormalizeDouble(Bid + TakeProfit * _Point, _Digits);
  		   	else TP = 0;

            if (TradeBlock) // Virtual Entry
            {
               VirtualDirection = POSITION_TYPE_BUY;
               VirtualOpen = true;
               VirtualOP = Ask;
               VirtualSL = SL;
               if (!Mute) Print("Entered Virtual Long at ", VirtualOP, " with SL at", VirtualSL);
               return;
            }

  				Trade.PositionOpen(_Symbol, ORDER_TYPE_BUY, Lots, Ask, SL, TP);
  				Sleep(1000);
  				if ((Trade.ResultRetcode() != 10008) && (Trade.ResultRetcode() != 10009) && (Trade.ResultRetcode() != 10010))
  					Print("Long Position Open Return Code: ", Trade.ResultRetcodeDescription());
  				else return;
  			}
   	}
   	// Sell condition
   	else if (((LastPosition == -1) && (random % 2 != 0)) || (LastPosition == POSITION_TYPE_BUY))
   	{
  			for (int i = 0; i < 10; i++)
  			{
  		   	Ask = SymbolInfoDouble(Symbol(), SYMBOL_ASK);
  				Bid = SymbolInfoDouble(Symbol(), SYMBOL_BID);
  				// Bid and Ask are swapped to preserve the probabilities and decrease/increase profit/loss size
  		      SL = NormalizeDouble(Ask + ATR[0], _Digits);
  				if (TakeProfit) TP = NormalizeDouble(Ask - TakeProfit * _Point, _Digits);
  				else TP = 0;

            if (TradeBlock) // Virtual Entry
            {
               VirtualDirection = POSITION_TYPE_SELL;
               VirtualOpen = true;
               VirtualOP = Bid;
               VirtualSL = SL;
               if (!Mute) Print("Entered Virtual Short at ", VirtualOP, " with SL at", VirtualSL);
               return;
            }
   
  				Trade.PositionOpen(_Symbol, ORDER_TYPE_SELL, Lots, Bid, SL, TP);
  				Sleep(1000);
  				if ((Trade.ResultRetcode() != 10008) && (Trade.ResultRetcode() != 10009) && (Trade.ResultRetcode() != 10010))
  					Print("Short Position Open Return Code: ", Trade.ResultRetcodeDescription());
  				else return;
  			}
   	}
	}
}

//+------------------------------------------------------------------+
//| Check What Position is Currently Open										|
//+------------------------------------------------------------------+
void GetPositionStates()
{
   if (TradeBlock) // Virtual Check
   {
      if (VirtualOpen)
      {
         if (VirtualDirection == POSITION_TYPE_BUY)
         {
   			HaveLongPosition = true;
			   HaveShortPosition = false;
         }
         else if (VirtualDirection == POSITION_TYPE_SELL)
         {
			   HaveLongPosition = false;
   			HaveShortPosition = true;
         }
      }
      else
      {
		   HaveLongPosition = false;
		   HaveShortPosition = false;
		}

      return;
   }

	// Is there a position on this currency pair?
	if (PositionInfo.Select(_Symbol))
	{
		if (PositionInfo.PositionType() == POSITION_TYPE_BUY)
		{
			HaveLongPosition = true;
			HaveShortPosition = false;
		}
		else if (PositionInfo.PositionType() == POSITION_TYPE_SELL)
		{ 
			HaveLongPosition = false;
			HaveShortPosition = true;
		}
   	if (HaveLongPosition) LastPosition = POSITION_TYPE_BUY;
   	else if (HaveShortPosition) LastPosition = POSITION_TYPE_SELL;
	}
	else 
	{
		HaveLongPosition = false;
		HaveShortPosition = false;
	}
}

//+------------------------------------------------------------------+
//| Close Open Position																|
//+------------------------------------------------------------------+
void ClosePrevious()
{
   if (TradeBlock) // Virtual Exit
   {
      if (VirtualOpen)
      {
         if (VirtualDirection == POSITION_TYPE_BUY)
         {
            double Bid = SymbolInfoDouble(Symbol(), SYMBOL_BID);
          	if (Bid < VirtualOP) TradeBlock = false;
            LastPosition = POSITION_TYPE_BUY;
            if (!Mute) Print("Closed Virtual Long at ", Bid, " with Open at", VirtualOP);
         }
         else if (VirtualDirection == POSITION_TYPE_SELL)
         {
          	double Ask = SymbolInfoDouble(Symbol(), SYMBOL_ASK);
            if (Ask > VirtualOP) TradeBlock = false;
            LastPosition = POSITION_TYPE_SELL;
            if (!Mute) Print("Closed Virtual Short at ", Ask, " with Open at", VirtualOP);
         }
         VirtualDirection = -1;
         VirtualOpen = false;
         VirtualOP = 0;
         VirtualSL = 0;
      }
      return;
   }

	for (int i = 0; i < 10; i++)
	{
		Trade.PositionClose(_Symbol, Slippage);
		if ((Trade.ResultRetcode() != 10008) && (Trade.ResultRetcode() != 10009) && (Trade.ResultRetcode() != 10010))
			Print("Position Close Return Code: ", Trade.ResultRetcodeDescription());
		else return;
	}
}

//+------------------------------------------------------------------+
//| Adjust Stop-Loss and TakeProfit of the Open Position					|
//+------------------------------------------------------------------+
void AdjustSLTP(double SLparam)
{
   if (TradeBlock) // Virtual Trailing Stop
   {
      if (VirtualOpen)
      {
         if (VirtualDirection == POSITION_TYPE_BUY)
         {
   		   double Bid = SymbolInfoDouble(Symbol(), SYMBOL_BID);
   			SL = NormalizeDouble(Bid - SLparam, _Digits);
   			if (SL > VirtualSL) VirtualSL = SL;
         }
         else if (VirtualDirection == POSITION_TYPE_SELL)
         {
   			double Ask = SymbolInfoDouble(Symbol(), SYMBOL_ASK);
   			SL = NormalizeDouble(Ask + SLparam, _Digits);
   			if ((SL < VirtualSL) || (VirtualSL == 0)) VirtualSL = SL;
         }
      }   
      return;
   }

	// Is there a position on this currency pair?
	if (PositionInfo.Select(_Symbol))
	{
		if (PositionInfo.PositionType() == POSITION_TYPE_BUY)
		{
		   double Bid = SymbolInfoDouble(Symbol(), SYMBOL_BID);
			SL = NormalizeDouble(Bid - SLparam/* * _Point*/, _Digits);
			TP = NormalizeDouble(PositionInfo.TakeProfit(), _Digits);
			if (SL > NormalizeDouble(PositionInfo.StopLoss(), _Digits))
			{
				for (int i = 0; i < 10; i++)
				{
					Trade.PositionModify(_Symbol, SL, TP);
					if ((Trade.ResultRetcode() != 10008) && (Trade.ResultRetcode() != 10009) && (Trade.ResultRetcode() != 10010))
						Print("Long Position Modify Return Code: ", Trade.ResultRetcodeDescription());
					else return;
				}
			}
		}
		else if (PositionInfo.PositionType() == POSITION_TYPE_SELL)
		{ 
			double Ask = SymbolInfoDouble(Symbol(), SYMBOL_ASK);
			SL = NormalizeDouble(Ask + SLparam/* * _Point*/, _Digits);
			TP = NormalizeDouble(PositionInfo.TakeProfit(), _Digits);
			if (SL < NormalizeDouble(PositionInfo.StopLoss(), _Digits))
			{
				for (int i = 0; i < 10; i++)
				{
					Trade.PositionModify(_Symbol, SL, TP);
					if ((Trade.ResultRetcode() != 10008) && (Trade.ResultRetcode() != 10009) && (Trade.ResultRetcode() != 10010))
						Print("Short Position Modify Return Code: ", Trade.ResultRetcodeDescription());
					else return;
				}
			}
		}
	}
}
//+------------------------------------------------------------------+

